/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "ExpCameraSpawner.h"

#include "MixCast.h"
#include "Data/SdkData.h"
#include "MixCastLog.h"

#include "Async/Async.h"

#include "Camera/CameraComponent.h"
#include "Engine/Engine.h"
#include "Engine/World.h"
#include "GameFramework/PlayerController.h"
#include "HeadMountedDisplay.h"
#include "Kismet/GameplayStatics.h"
#include "HeadMountedDisplay.h"

UExpCameraSpawner::UExpCameraSpawner(const FObjectInitializer &ObjectInitializer)
	: Super(ObjectInitializer)
{
}

void UExpCameraSpawner::SynchronizeSpawnedCameras()
{
	for (int i = cameras.Num() - 1; i >= 0; i--)
	{
		bool foundAgain = false;
		for (int j = 0; j < SdkData::VirtualCameras.size() && !foundAgain; j++)
			foundAgain |= SdkData::VirtualCameras[j].identifier.compare(cameras[i]->Identifier) == 0;

		if (!foundAgain)
		{
			if (cameras[i]->IsValidLowLevel())
				cameras[i]->Destroy();
			cameras.RemoveAt(i);
		}
	}

	for (int i = 0; i < SdkData::VirtualCameras.size(); i++)
	{
		bool alreadyFound = false;
		for (int j = 0; j < cameras.Num() && !alreadyFound; j++)
			alreadyFound |= cameras[j]->Identifier.compare(SdkData::VirtualCameras[i].identifier) == 0;

		if (!alreadyFound)
		{
			UClass* camClass = SdkData::GetProjectSettings()->CameraClass.TryLoadClass<AExpCamera>();
			if (camClass == nullptr)
				camClass = AExpCamera::StaticClass();
			AActor* camActor = UGameplayStatics::BeginDeferredActorSpawnFromClass(this, camClass, FTransform::Identity);
			AExpCamera* cam = Cast<AExpCamera>(camActor);
			cam->Identifier = SdkData::VirtualCameras[i].identifier;
			cam->GetRootComponent()->AttachToComponent(this, FAttachmentTransformRules::KeepRelativeTransform);
			UGameplayStatics::FinishSpawningActor(camActor, FTransform::Identity);
			cameras.Add(cam);
		}
	}
}

void UExpCameraSpawner::ClearSpawnedCameras()
{
	for (int i = 0; i < cameras.Num(); i++)
	{
		if (cameras[i]->IsValidLowLevel())
			cameras[i]->Destroy();
	}
	cameras.Empty();
}

void UExpCameraSpawner::RenderCameras()
{
	for (auto camera : cameras)
		camera->CaptureIfNeeded();
}
