/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "SdkCustomTrackedObjectMediator.h"

#include "MixCast.h"
#include "Transmission/Thrift/ServiceCommunicator.h"
#include "Utility/StringUtils.hpp"

#include "GameFramework/WorldSettings.h"

void SdkCustomTrackedObjectMediator::UpdateClientIfChanged(FServiceCommunicator* communicator)
{
	bool listChanged = false;

	while (data.size() < UCustomTrackedObjectComponent::Active.Num())
	{
		data.push_back(TrackedObject());
		listChanged = true;
	}
	while (data.size() > UCustomTrackedObjectComponent::Active.Num())
	{
		data.erase(data.begin() + data.size() - 1);
		listChanged = true;
	}

	for (int i = 0; i < data.size(); i++)
	{
		listChanged |= UpdateTrackedObjectData(UCustomTrackedObjectComponent::Active[i], &data[i]);
	}

	if (listChanged)
	{
		communicator->UpdateExperienceTrackedObjects(data);
	}
}

bool SdkCustomTrackedObjectMediator::UpdateTrackedObjectData(UCustomTrackedObjectComponent* objComp, TrackedObject* objData)
{
	bool changed = false;

	if (!StringUtils::StdAndFStrAreEqual(objData->identifier, objComp->ObjectIdentifier)) {
		objData->identifier = TCHAR_TO_UTF8(*objComp->ObjectIdentifier);
		changed = true;
	}

	if (!StringUtils::StdAndFStrAreEqual(objData->name, objComp->ObjectName)) {
		objData->name = TCHAR_TO_UTF8(*objComp->ObjectName);
		changed = true;
	}

	objData->connected = true;
	objData->objectType = UCustomTrackedObjectComponent::UnrealObjectTypeToThrift(objComp->ObjectType);
	objData->assignedRole = UCustomTrackedObjectComponent::UnrealAssignedRoleToThrift(objComp->ObjectRole);
	objData->hideFromUser = objComp->HideFromUser;

	float worldToMeters = objComp->GetWorld()->GetWorldSettings()->WorldToMeters;

	FVector newPos = objComp->GetSharedPosition();
	FVector oldPos = ThriftUtil::ThriftPositionToUnreal(objData->position, worldToMeters);
	if (FVector::DistSquared(oldPos, newPos) > 0.1f * 0.1f)
	{
		objData->position = ThriftUtil::UnrealVectorToThrift(newPos, worldToMeters);
		changed = true;
	}

	FQuat newRot = objComp->GetSharedRotation();
	FQuat oldRot = ThriftUtil::ThriftQuatToUnreal(objData->rotation);
	if (oldRot.AngularDistance(newRot) > 0.01)
	{
		objData->rotation = ThriftUtil::UnrealQuatToThrift(newRot);
		changed = true;
	}
	
	return changed;
}