/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "TrackingUtil.h"

mixcast::data::TrackingSource::type TrackingUtil::TrackingTypeOverride = mixcast::data::TrackingSource::UNKNOWN;

void TrackingUtil::SetOverrideTrackingType(mixcast::data::TrackingSource::type type)
{
	TrackingTypeOverride = type;
}

mixcast::data::TrackingSource::type TrackingUtil::GetCurrentTrackingType(bool ignoreOverride)
{
	if (!ignoreOverride && TrackingTypeOverride != mixcast::data::TrackingSource::UNKNOWN)
		return TrackingTypeOverride;

#if UE_4_18_OR_LATER
	if (!GEngine || !GEngine->XRSystem.IsValid() || !GEngine->StereoRenderingDevice.IsValid())
		return mixcast::data::TrackingSource::UNKNOWN;

	IHeadMountedDisplay* HMD = GEngine->XRSystem->GetHMDDevice();
	if (HMD == NULL || !HMD->IsHMDConnected() || !HMD->IsHMDEnabled() || !GEngine->StereoRenderingDevice->IsStereoEnabled())
		return mixcast::data::TrackingSource::UNKNOWN;

	FName sysName = GEngine->XRSystem->GetSystemName();
	if (sysName.IsEqual(TEXT("SteamVR")))
		return mixcast::data::TrackingSource::STEAMVR;
	else if (sysName.IsEqual(TEXT("OculusHMD")))
		return mixcast::data::TrackingSource::OCULUS;
#if MIXCAST_OPENXR
	else if (sysName.IsEqual(TEXT("OpenXR")))
		return mixcast::data::TrackingSource::OPENXR;
#endif
	else
		return mixcast::data::TrackingSource::UNKNOWN;
#else
	if (!GEngine || !GEngine->HMDDevice.IsValid())
		return mixcast::data::TrackingSource::UNKNOWN;

	switch (GEngine->HMDDevice->GetHMDDeviceType())
	{
	case EHMDDeviceType::DT_SteamVR:
		return mixcast::data::TrackingSource::STEAMVR;
	case EHMDDeviceType::DT_OculusRift:
		return mixcast::data::TrackingSource::OCULUS;
	default:
		return mixcast::data::TrackingSource::UNKNOWN;
	}
#endif
}
