/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "SharedTextureReceiver.h"
#include "MixCast.h"
#include "Transmission/Thrift/ServiceCommunicator.h"
#include "Utility/StringUtils.hpp"

#include "Engine/Texture2D.h"

using namespace mixcast::data;
using namespace mixcast::thrift;

SharedTextureReceiver::SharedTextureReceiver(FString texId) :
	SharedTexWatcher(NULL)
{
	TextureId = texId;

	FMixCast* mod = ((FMixCast*)&IMixCast::Get());

	EventHandle = mod->OnExternalTextureUpdated().AddRaw(this, &SharedTextureReceiver::RefreshTextureFromEvent);

	TextureInfo = mod->GetClientConnection()->GetSharedTextureInfo(TextureId);
	if (TextureInfo.handle == 0)
		return;

	RefreshTextureObject();
}

SharedTextureReceiver::~SharedTextureReceiver()
{
	FMixCast* mod = ((FMixCast*)&IMixCast::Get());
	if (mod)
		mod->OnExternalTextureUpdated().Remove(EventHandle);

	DestroyTexture();
	TextureReferenceChanged.Broadcast();
}

void SharedTextureReceiver::EnqueueUpdate()
{
	if (SharedTexWatcher == NULL)
		return;

	SharedTexWatcher->EnqueueRead();
}

void SharedTextureReceiver::RefreshTextureObject()
{
	DestroyTexture();

	if (TextureInfo.handle == 0 || TextureInfo.width == 0 || TextureInfo.height == 0) {
		TextureReferenceChanged.Broadcast();
		return;
	}

	if (TextureInfo.procId != 0) {
		srcProcHandle = OpenProcess(PROCESS_DUP_HANDLE, true, TextureInfo.procId);
		SharedTexWatcher = WatchedUnrealSharedTexture::Create(srcProcHandle, (HANDLE)TextureInfo.handle);
	}
	else
		SharedTexWatcher = WatchedUnrealSharedTexture::Create((HANDLE)TextureInfo.handle);

	TextureReferenceChanged.Broadcast();
}

void SharedTextureReceiver::RefreshTextureFromEvent(std::string TexId, SharedTex TexInfo)
{
	if (StringUtils::StdAndFStrAreEqual(TexId, TextureId))
	{
		TextureInfo = TexInfo;
		RefreshTextureObject();
	}
}

void SharedTextureReceiver::DestroyTexture()
{
	//FlushRenderingCommands();
	if (SharedTexWatcher)
	{
		SharedTexWatcher->BeginDestroy();
		SharedTexWatcher = NULL;
	}
	CloseHandle(srcProcHandle);
	srcProcHandle = 0;
}
