#pragma once

#include "MixCastDefines.h"
#include "../../../MixCastLog.h"

#include "CoreMinimal.h"
#include "RHICommandList.h"
#include "Engine/Texture2D.h"
#if UE_4_26_OR_LATER
#include "Rendering/Texture2DResource.h"
#else
#include "TextureResource.h"
#endif

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif
#include <Windows.h>
#include <stdio.h>
#include <conio.h>
#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif

class WatchedUnrealSharedTexture
{
protected:
	UTexture2D* dstTex = NULL;

	virtual void Read_RenderThread(FRHICommandListImmediate& RHICmdList) = 0;

	static void* GetNativeTexture(UTexture2D* tex)
	{
#if UE_4_26_OR_LATER
		FTextureResource* res = (FTextureResource*)tex->Resource;
#else
		FTexture2DResource* res = (FTexture2DResource*)tex->Resource;
#endif
		if (res == NULL)
			return NULL;
		FTexture2DRHIRef rhiRef = res->GetTexture2DRHI();
		if (!rhiRef.IsValid())
			return NULL;
		return rhiRef->GetNativeResource();
	}

	virtual ~WatchedUnrealSharedTexture() {}

public:
	static EPixelFormat GetUnrealFormatFromDirectXFormat(unsigned int format);

	static WatchedUnrealSharedTexture* Create(HANDLE remoteTexHandle);
	static WatchedUnrealSharedTexture* Create(HANDLE srcProcId, HANDLE remoteTexHandle);

	void EnqueueRead()
	{
		ENQUEUE_RENDER_COMMAND(void)(
			[&](FRHICommandListImmediate& RHICmdList)
			{
				Read_RenderThread(RHICmdList);
			});
	}
	virtual void BeginDestroy() {}

	UTexture2D* GetUnrealTex()
	{
		return dstTex;
	}
};