#pragma once

#include "WatchedUnrealSharedTexture_D3D12.h"
#include "../../../MixCastLog.h"

#if MIXCAST_D3D12

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif
#include <d3d12.h>
#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif

#include "ITextureShareD3D12.h"

void WatchedUnrealSharedTexture_D3D12::Initialize(HANDLE texHandle)
{
	ID3D12Device* device = (ID3D12Device*)GDynamicRHI->RHIGetNativeDevice();

	HRESULT hr = device->OpenSharedHandle(localHandle, IID_PPV_ARGS(&sharedTex));

	if (FAILED(hr))
		return;

	D3D12_RESOURCE_DESC refDesc = sharedTex->GetDesc();

	width = refDesc.Width;
	height = refDesc.Height;

	if (!ITextureShareD3D12::Get().CreateRHITexture(sharedTex, GetUnrealFormatFromDirectXFormat(refDesc.Format), sharedTexRhi))
		MIXCAST_ERROR("Couldn't create RHI for shared watched texture");

	dstTex = UTexture2D::CreateTransient(refDesc.Width, refDesc.Height, GetUnrealFormatFromDirectXFormat(refDesc.Format));
	dstTex->AddToRoot();
	dstTex->UpdateResource();
}
void WatchedUnrealSharedTexture_D3D12::BeginDestroy()
{
	SAFE_RELEASE(sharedTex);
	CloseHandle(localHandle);

	if (dstTex && dstTex->IsValidLowLevel())
	{
		if (dstTex->Resource)
			dstTex->ReleaseResource();
		dstTex->RemoveFromRoot();
		dstTex->MarkPendingKill();
	}
	dstTex = nullptr;

	ENQUEUE_RENDER_COMMAND(void)(
		[&](FRHICommandListImmediate& RHICmdList)
		{
			delete this;
		});
}

void WatchedUnrealSharedTexture_D3D12::Read_RenderThread(FRHICommandListImmediate& RHICmdList)
{
	if (dstTex == NULL || sharedTex == NULL || !dstTex->IsValidLowLevel())
		return;

	FBox2D CopyBox(FVector2D(0.0f, 0.0f), FVector2D(dstTex->GetSurfaceWidth(), dstTex->GetSurfaceHeight()));
	RHICmdList.CopySubTextureRegion(sharedTexRhi, dstTex->Resource->GetTexture2DRHI(), CopyBox, CopyBox);
}
#endif