/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"

class MIXCAST_API MathUtil
{
public:
	static inline FVector MixCastVectorToUnreal(const FVector& MixCastVec)
	{
		return {
			MixCastVec.Z,
			MixCastVec.X,
			MixCastVec.Y,
		};
	}
	static inline FVector UnrealVectorToMixCast(const FVector& MixCastVec)
	{
		return {
			MixCastVec.Y,
			MixCastVec.Z,
			MixCastVec.X,
		};
	}

	static inline FVector MixCastPositionToUnreal(const FVector &MixCastPosition, float WorldToMeters)
	{
		return MixCastVectorToUnreal(MixCastPosition) * WorldToMeters;
	}
	static inline FVector UnrealPositionToMixCast(const FVector &UnrealPosition, float WorldToMeters)
	{
		return UnrealVectorToMixCast(UnrealPosition) / WorldToMeters;
	}

	static inline FQuat MixCastRotationToUnreal(const FQuat &MixCastRotation)
	{
		return {
			MixCastRotation.Z,
			MixCastRotation.X,
			MixCastRotation.Y,
			MixCastRotation.W,
		};
	}
	static inline FQuat UnrealRotationToMixCast(const FQuat &UnrealRotation)
	{
		return {
			UnrealRotation.Y,
			UnrealRotation.Z,
			UnrealRotation.X,
			UnrealRotation.W
		};
	}

	//degrees in/out
	static inline float VerticalFoVToHorizontal(const float vertFoV, const float aspectRatio)
	{
		const float vFovRadians = vertFoV * PI / 180.0f;
		const float hFovRadians = atan(tan(vFovRadians / 2.0f) * aspectRatio) * 2.0f;
		return hFovRadians / PI * 180.0f;
	}
};
