/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include <string>

#include "CoreMinimal.h"
#include "Containers/UnrealString.h"
#include "Misc/ConfigCacheIni.h"
#include "Engine/Engine.h"

#include "MixCast/Public/IMixCast.h"

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif

class MIXCAST_API StringUtils
{
public:
	static std::string ToStdString(const FString& String)
	{
		return std::string(TCHAR_TO_UTF8(*String));
	}
	static std::wstring ToStdWString(const FString& String)
	{
		std::string s = ToStdString(String);

		int len;
		int slength = (int)s.length() + 1;
		len = MultiByteToWideChar(CP_ACP, 0, s.c_str(), slength, 0, 0);
		wchar_t* buf = new wchar_t[len];
		MultiByteToWideChar(CP_ACP, 0, s.c_str(), slength, buf, len);
		std::wstring ws(buf);
		delete[] buf;

		return ws;
	}

	static bool StdAndFStrAreEqual(const std::string& stdStr, const FString& fStr)
	{
		if (stdStr.length() != fStr.Len())
			return false;

		int fStrCharIndex = 0;
		for (char const& c : stdStr)
		{
			if (c != fStr[fStrCharIndex++])
				return false;
		}
		return true;
	}

	static bool IsVersionBLaterThanVersionA(FString versionA, FString versionB)
	{
		if (versionA.Compare(versionB) == 0)
			return false;

		TArray<FString> aNums, bNums;
		versionA.ParseIntoArray(aNums, TEXT("."));
		versionB.ParseIntoArray(bNums, TEXT("."));

		for (int i = 0; i < aNums.Num() && i < bNums.Num(); i++)
		{
			int aNum = FCString::Atoi(*aNums[i]);
			int bNum = FCString::Atoi(*bNums[i]);

			if (bNum > aNum)
				return true;
			else if (bNum < aNum)
				return false;
		}

		if (bNums.Num() > aNums.Num())
			return true;

		return false;
	}
};

#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif