/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "MixCast_OpenXR.h"
#if MIXCAST_OPENXR
#include "MixCast_OpenXRLog.h"

#include "MixCast.h"
#include "MixCastSdkActor.h"
#include "MixCast/Private/Tracking/TrackingUtil.h"

#include "CoreMinimal.h"
#include "GameFramework/WorldSettings.h"
#include "HeadMountedDisplayFunctionLibrary.h"

#include "IXRTrackingSystem.h"
#include "OpenXRCore.h"
#include "OpenXRHMD/Private/OpenXRHMD.h"
#endif

IMPLEMENT_MODULE(FMixCast_OpenXR, MixCast_OpenXR)

#if MIXCAST_OPENXR
void FMixCast_OpenXR::StartupModule()
{
	IMixCast& mod = IMixCast::Get();
	beforeFrameDelegate = mod.OnBeforeFrame().AddRaw(this, &FMixCast_OpenXR::HandleMixCastAboutToRender);
}
void FMixCast_OpenXR::ShutdownModule()
{
	if (IMixCast::IsAvailable()) {
		IMixCast& mod = IMixCast::Get();
		mod.OnBeforeFrame().Remove(beforeFrameDelegate);
	}
}
void FMixCast_OpenXR::HandleMixCastAboutToRender()
{
	if (TrackingUtil::GetCurrentTrackingType(true) != TrackingSource::OPENXR)
		return;

	FTransform hmdToWorld, trackingOriginToHmd;
	bool success = CalculateHmdToWorldTransform(hmdToWorld);
	if (!success)
		return;

	success = CalculateTrackingOriginToHmdTransform(trackingOriginToHmd);
	if (!success)
		return;

	FTransform playAreaToWorld = trackingOriginToHmd.Inverse() * hmdToWorld;

	AMixCastSdkActor::SetWorldTransform(playAreaToWorld);
}

bool FMixCast_OpenXR::CalculateHmdToWorldTransform(FTransform& trans)
{
	if (!GEngine->XRSystem.IsValid())
		return false;

	trans = GEngine->XRSystem->GetTrackingToWorldTransform();
	return true;
}

bool FMixCast_OpenXR::CalculateTrackingOriginToHmdTransform(FTransform& trans)
{
	trans = FTransform::Identity;	//can get the tracking origin on its own now
	return true;
}

bool FMixCast_OpenXR::CalculateTrackingOriginToStandingTransform(FTransform& trans)
{
	EHMDTrackingOrigin::Type curOrigin = GEngine->XRSystem->GetTrackingOrigin();
	if (curOrigin == EHMDTrackingOrigin::Stage)
	{
		trans = FTransform::Identity;
		return true;
	}

	FOpenXRHMD* hmd = (FOpenXRHMD*)GEngine->XRSystem->GetHMDDevice();
	XrSpace localSpace = hmd->GetTrackingSpace();
	GEngine->XRSystem->SetTrackingOrigin(EHMDTrackingOrigin::Stage);
	XrSpace stageSpace = hmd->GetTrackingSpace();
	GEngine->XRSystem->SetTrackingOrigin(curOrigin);

	XrSpaceLocation loc = {};
	XR_ENSURE(xrLocateSpace(localSpace, stageSpace, hmd->GetDisplayTime(), &loc));
	trans.SetComponents(FQuat(loc.pose.orientation.x, loc.pose.orientation.y, loc.pose.orientation.z, loc.pose.orientation.w),
		FVector(loc.pose.position.x, loc.pose.position.y, loc.pose.position.z),
		FVector::OneVector);
	return true;
}
#endif	//MIXCAST_OPENXR