/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "ClearPostOpaque.h"
#if MIXCAST_SHADER_PASSES
#include "CommonRenderResources.h"
#include "Containers/DynamicRHIResourceArray.h"
#include "Runtime/Renderer/Private/PostProcess/SceneRenderTargets.h"

class FClearPostOpaqueVertexBuffer : public FVertexBuffer
{
public:
	/** Initialize the RHI for this rendering resource */
	void InitRHI()
	{
		TResourceArray<FFilterVertex, VERTEXBUFFER_ALIGNMENT> Vertices;
		Vertices.SetNumUninitialized(6);

		Vertices[0].Position = FVector4(-1, 1, 0, 1);
		Vertices[0].UV = FVector2D(0, 0);

		Vertices[1].Position = FVector4(1, 1, 0, 1);
		Vertices[1].UV = FVector2D(1, 0);

		Vertices[2].Position = FVector4(-1, -1, 0, 1);
		Vertices[2].UV = FVector2D(0, 1);

		Vertices[3].Position = FVector4(1, -1, 0, 1);
		Vertices[3].UV = FVector2D(1, 1);

		// Create vertex buffer. Fill buffer with initial data upon creation
		FRHIResourceCreateInfo CreateInfo(&Vertices);
		VertexBufferRHI = RHICreateVertexBuffer(Vertices.GetResourceDataSize(), BUF_Static, CreateInfo);
	}
};
TGlobalResource<FClearPostOpaqueVertexBuffer> GClearPostOpaqueVertexBuffer;

class FMixCastPostOpaqueGrabVS : public FGlobalShader
{
	DECLARE_GLOBAL_SHADER(FMixCastPostOpaqueGrabVS);

public:
	FMixCastPostOpaqueGrabVS() { }
	FMixCastPostOpaqueGrabVS(const ShaderMetaType::CompiledShaderInitializerType& Initializer)
		: FGlobalShader(Initializer)
	{ }

	static bool ShouldCompilePermutation(const FGlobalShaderPermutationParameters& Parameters)
	{
		return true;
	}
};

template<int32 NumOutputs>
class FMixCastPostOpaqueGrabPS : public FGlobalShader
{
public:
	DECLARE_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS);

	static bool ShouldCompilePermutation(const FGlobalShaderPermutationParameters& Parameters)
	{
		return IsFeatureLevelSupported(Parameters.Platform, ERHIFeatureLevel::SM5);
	}

	static void ModifyCompilationEnvironment(const FGlobalShaderPermutationParameters& Parameters, FShaderCompilerEnvironment& OutEnvironment)
	{
		FGlobalShader::ModifyCompilationEnvironment(Parameters, OutEnvironment);
		OutEnvironment.SetDefine(TEXT("NUM_OUTPUTS"), NumOutputs);
	}

public:
	FMixCastPostOpaqueGrabPS() { }
	FMixCastPostOpaqueGrabPS(const ShaderMetaType::CompiledShaderInitializerType& Initializer)
		: FGlobalShader(Initializer)
	{
	}

	 void SetParameters(FRHICommandList& RHICmdList, TShaderMapRef<FMixCastPostOpaqueGrabPS> ShaderRef, FSceneView& InView)
	 {
		 FRHISamplerState* SamplerStateRHI = TStaticSamplerState<SF_Bilinear>::GetRHI();
 #if !UE_4_25_OR_LATER
		 FRHIPixelShader* ShaderRHI = GetPixelShader();
 #else
		 FRHIPixelShader* ShaderRHI = ShaderRef.GetPixelShader();
 #endif

		 FGlobalShader::SetParameters<FViewUniformShaderParameters>(RHICmdList, ShaderRHI, InView.ViewUniformBuffer);
	 }
};

IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabVS, "/Plugin/MixCast/ClearPostOpaque.usf", "MainVS", SF_Vertex);

IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS<1>, "/Plugin/MixCast/ClearPostOpaque.usf", "MainPS", SF_Pixel);
IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS<2>, "/Plugin/MixCast/ClearPostOpaque.usf", "MainPS", SF_Pixel);
IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS<3>, "/Plugin/MixCast/ClearPostOpaque.usf", "MainPS", SF_Pixel);
IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS<4>, "/Plugin/MixCast/ClearPostOpaque.usf", "MainPS", SF_Pixel);
IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS<5>, "/Plugin/MixCast/ClearPostOpaque.usf", "MainPS", SF_Pixel);
IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS<6>, "/Plugin/MixCast/ClearPostOpaque.usf", "MainPS", SF_Pixel);
IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS<7>, "/Plugin/MixCast/ClearPostOpaque.usf", "MainPS", SF_Pixel);
IMPLEMENT_GLOBAL_SHADER(FMixCastPostOpaqueGrabPS<8>, "/Plugin/MixCast/ClearPostOpaque.usf", "MainPS", SF_Pixel);


void FMixCastShader_ClearPostOpaque::Execute(FRHICommandListImmediate& RHICmdList, FSceneView& InView)
{
	FSceneRenderTargets& SceneContext = FSceneRenderTargets::Get(RHICmdList);

	if (RHICmdList.IsInsideRenderPass())
		RHICmdList.EndRenderPass();

#if UE_4_25_OR_LATER
	FRHIRenderPassInfo RPInfo;
	int32 velIndex = -1;
#if UE_4_26_OR_LATER
	SceneContext.FillGBufferRenderPassInfo(ERenderTargetLoadAction::ELoad, RPInfo, velIndex);
#else
	int32 tangentIndex = -1;
	SceneContext.FillGBufferRenderPassInfo(ERenderTargetLoadAction::ELoad, RPInfo, velIndex, tangentIndex);
#endif
	RPInfo.DepthStencilRenderTarget.DepthStencilTarget = SceneContext.GetSceneDepthSurface();
	RPInfo.DepthStencilRenderTarget.ExclusiveDepthStencil = FExclusiveDepthStencil::DepthWrite_StencilNop;
	RPInfo.DepthStencilRenderTarget.Action = MakeDepthStencilTargetActions(ERenderTargetActions::Load_Store, ERenderTargetActions::DontLoad_DontStore);
	RHICmdList.BeginRenderPass(RPInfo, TEXT("MixCast-ClearPostOpaque"));
#else
	SceneContext.BeginRenderingGBuffer(RHICmdList, ERenderTargetLoadAction::ELoad, ERenderTargetLoadAction::ELoad, FExclusiveDepthStencil::DepthWrite_StencilNop, false);
#endif

	auto ShaderMap = GetGlobalShaderMap(InView.GetFeatureLevel());

	TShaderMapRef<FMixCastPostOpaqueGrabVS> VertexShader(ShaderMap);

	//prepare state
	FGraphicsPipelineStateInitializer GraphicsPSOInit;
	RHICmdList.ApplyCachedRenderTargets(GraphicsPSOInit);

	GraphicsPSOInit.BlendState = TStaticBlendState<>::GetRHI();
	GraphicsPSOInit.RasterizerState = TStaticRasterizerState<>::GetRHI();
	GraphicsPSOInit.DepthStencilState = TStaticDepthStencilState<false, CF_Always>::GetRHI();

	GraphicsPSOInit.BoundShaderState.VertexDeclarationRHI = GFilterVertexDeclaration.VertexDeclarationRHI;
	GraphicsPSOInit.PrimitiveType = PT_TriangleStrip;
#if UE_4_25_OR_LATER
	GraphicsPSOInit.BoundShaderState.VertexShaderRHI = VertexShader.GetVertexShader();
#else
	GraphicsPSOInit.BoundShaderState.VertexShaderRHI = GETSAFERHISHADER_VERTEX(*VertexShader);
#endif

	//set pixel shader
	uint32 NumActiveRenderTargets = SceneContext.GetNumGBufferTargets();
	switch (NumActiveRenderTargets)
	{
	case 5:
	{
		TShaderMapRef<FMixCastPostOpaqueGrabPS<5>> MRTPixelShader(ShaderMap);
#if UE_4_25_OR_LATER
		GraphicsPSOInit.BoundShaderState.PixelShaderRHI = MRTPixelShader.GetPixelShader();
#else
		GraphicsPSOInit.BoundShaderState.PixelShaderRHI = GETSAFERHISHADER_PIXEL(*MRTPixelShader);
#endif

		SetGraphicsPipelineState(RHICmdList, GraphicsPSOInit);

		MRTPixelShader->SetParameters(RHICmdList, MRTPixelShader, InView);
	}
	break;
	case 6:
	{
		TShaderMapRef<FMixCastPostOpaqueGrabPS<6>> MRTPixelShader(ShaderMap);
#if UE_4_25_OR_LATER
		GraphicsPSOInit.BoundShaderState.PixelShaderRHI = MRTPixelShader.GetPixelShader();
#else
		GraphicsPSOInit.BoundShaderState.PixelShaderRHI = GETSAFERHISHADER_PIXEL(*MRTPixelShader);
#endif

		SetGraphicsPipelineState(RHICmdList, GraphicsPSOInit);

		MRTPixelShader->SetParameters(RHICmdList, MRTPixelShader, InView);
	}
	break;
	default:
	case 1:
	{
		TShaderMapRef<FMixCastPostOpaqueGrabPS<1>> MRTPixelShader(ShaderMap);
#if UE_4_25_OR_LATER
		GraphicsPSOInit.BoundShaderState.PixelShaderRHI = MRTPixelShader.GetPixelShader();
#else
		GraphicsPSOInit.BoundShaderState.PixelShaderRHI = GETSAFERHISHADER_PIXEL(*MRTPixelShader);
#endif

		SetGraphicsPipelineState(RHICmdList, GraphicsPSOInit);

		MRTPixelShader->SetParameters(RHICmdList, MRTPixelShader, InView);
	}
	break;
	}

	FPooledRenderTargetDesc Desc = SceneContext.GetSceneColor()->GetDesc();

	//dispatch draw
	RHICmdList.SetViewport(0, 0, 0, Desc.Extent.X, Desc.Extent.Y, 1);
	RHICmdList.SetStreamSource(0, GClearPostOpaqueVertexBuffer.VertexBufferRHI, 0);
	RHICmdList.DrawPrimitive(0, 2, 1);

#if UE_4_26_OR_LATER
	RHICmdList.EndRenderPass();
#endif
}
#endif