/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "MixCastShaders.h"
#include "WriteOcclusion.h"
#include "ReadOpaque.h"
#include "ReadDepth.h"
#include "ReadAlpha.h"
#include "ClearPostOpaque.h"
#include "RewriteOpaque.h"

#include "Interfaces/IPluginManager.h"
#include "Containers/DynamicRHIResourceArray.h"
#include "Runtime/Renderer/Private/PostProcess/SceneRenderTargets.h"
#include "ShaderParameterUtils.h"

IMPLEMENT_MODULE(FMixCastShaders, MixCast_Shaders)

void FMixCastShaders::StartupModule()
{
#if UE_4_22_OR_LATER
	FString PluginShaderDir = FPaths::Combine(IPluginManager::Get().FindPlugin("MixCast")->GetBaseDir(), TEXT("Shaders"));
	AddShaderSourceDirectoryMapping(TEXT("/Plugin/MixCast"), PluginShaderDir);
#endif
}
void FMixCastShaders::ShutdownModule()
{

}

void FMixCastShaders::ReadAlpha(
	IRendererModule& RendererModule,
	FRHICommandListImmediate& RHICmdList,
	FSceneRenderTargets& SceneContext,
	FSceneView& SceneView,
	UTextureRenderTarget2D& AlphaTarget)
{
	FMixCastShader_ReadAlpha::Execute(RendererModule, RHICmdList, SceneContext, SceneView, AlphaTarget);
}

#if MIXCAST_SHADER_PASSES
void FMixCastShaders::ReadOpaque(IRendererModule& RendererModule,	FPostOpaqueRenderParameters& Parameters, UTextureRenderTarget2D& OpaqueLayerTarget, bool ReadDepth)
{
	FMixCastShader_ReadOpaque::Execute(RendererModule, Parameters, OpaqueLayerTarget, ReadDepth);
}
void FMixCastShaders::ReadDepth(IRendererModule& RendererModule, FRHICommandList& RHICmdList, FSceneView& InView, UTextureRenderTarget2D& OpaqueLayerTarget)
{
	FMixCastShader_ReadDepth::Execute(RendererModule, RHICmdList, InView, OpaqueLayerTarget);
}

void FMixCastShaders::WriteOcclusion(FRHICommandListImmediate& RHICmdList, FSceneView& InView, bool basePass, FRHITexture* DepthTextureRHI, float DepthMultiplierValue)
{
	FMixCastShader_WriteOcclusion::Execute(RHICmdList, InView, basePass, DepthTextureRHI, DepthMultiplierValue);
}
void FMixCastShaders::ClearPostOpaque(FRHICommandListImmediate& RHICmdList, FSceneView& InView)
{
	FMixCastShader_ClearPostOpaque::Execute(RHICmdList, InView);
}
void FMixCastShaders::RewriteOpaque(FRHICommandListImmediate& RHICmdList, FSceneView& InView, FRHITexture* OpaqueTextureRHI)
{
	FMixCastShader_RewriteOpaque::Execute(RHICmdList, InView, OpaqueTextureRHI);
}
#endif